#include <OneWire.h>
#include <DallasTemperature.h>//important for the temperature-sensor
#include <EEPROM.h>//non vioitale memory
#include <Wire.h>  // Include Wire if you're using I2C
#include <SPI.h>  // Include SPI if you're using SPI
#include <SFE_MicroOLED.h>  // Include the SFE_MicroOLED library

//Pin Assignment
#define ONE_WIRE_BUS 8  // Temperature Takt is on Pin 
#define PIN_RESET 9 // Connect OLED Display RST to 
#define PIN_DC    5  // Connect OLED Display DC to 
#define PIN_CS    4 // Connect OLED Display CS to 
#define DC_JUMPER 1
#define PIN_pushButton 3//Connect Pin to Button
#define PIN_LED 13//Connect Pin to LED

//Variables
const char Version[4] = "1.5";
volatile double Takt, Tmax, Tmin;
volatile byte record = 0;
volatile int EEAdress = 0; //Address for the EEProm 
char SensOK =0; //=1->Sensor is ok
String data;

//Instancing Objects
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
DeviceAddress Thermometer; // arrays to hold device address
MicroOLED oled(PIN_RESET, DC_JUMPER);    // I2C declaration


void setup(void) {
  pinMode(ONE_WIRE_BUS, INPUT_PULLUP);          
  Serial.begin(9600);

  oled.begin();    // Initialize the OLED
  oled.clear(ALL); // Clear the display's internal memory
  oled.display();  // Display what's in the buffer (splashscreen)
  delay(500);     // Delay 1000 ms
  oled.clear(PAGE); // Clear the buffer.
  pinMode(PIN_LED, OUTPUT); 
  pinMode(PIN_pushButton, INPUT_PULLUP);
  
  sensors.begin();     //Start Library
  if (!sensors.getAddress(Thermometer, 0)) {
    Serial.println("Unable to find address for Device 0"); //Very important!
  }
  else {
    SensOK=1;
  }
  sensors.setResolution(Thermometer,12);  
  delay(1000);     // Delay 1000 ms
  Takt = sensors.getTempCByIndex(0); //Set Input to Current Temperature
  Tmax = -1000;
  Tmin = Takt;

  //Serial.print("TempLogger V");
  //Serial.println(Version);
  oled.setCursor(0, 0);        // Set cursor to top-left
  oled.setFontType(0);         
  oled.print("TempLogger");      
  oled.setCursor(0, 20);        // Set cursor to top-left
  oled.setFontType(2);
  oled.print(Version);         
  oled.display();
  delay(4000);
  oled.clear(PAGE); // Clear the buffer. 
  attachInterrupt(1, clicked, FALLING);    // Click button on interrupt 1 - pin 3
  record = 0;
}

void loop(void) {
  if (SensOK==1){
    sensors.requestTemperatures();
    Takt=(double)(sensors.getTempCByIndex(0)); 
  
    digitalWrite(PIN_LED, record); //Control LED
  
    if (record!=0) {
      if (EEAdress<1024) EEAdress++;
      EEPROM.write(EEAdress,(unsigned char)Takt);
      Serial.print(EEAdress);
      Serial.print(";");
    }
    else
    {
      if (EEAdress!=0){
        if (EEAdress<1024) EEAdress++;
        EEPROM.write(EEAdress,(unsigned char)0xFF);  
        EEAdress=0;
      }
    }
      if (Tmin>Takt) Tmin=Takt;
      if (Tmax<Takt) Tmax=Takt;
      Serial.println(Takt);
  
      oled.clear(PAGE);            // Clear the display
      oled.setCursor(0, 0);        // Set cursor to top-left
      oled.setFontType(0);         // Smallest font
      oled.print("min:");        
      oled.setFontType(0);       
      oled.print(Tmin);  
      oled.setCursor(0, 10);       // Set cursor to top-middle-left
      oled.setFontType(0);         // Repeat
      oled.print("akt:");
      oled.setFontType(0);
      oled.print(Takt);
      oled.setCursor(0, 20);
      oled.setFontType(0);
      oled.print("max:");
      oled.setFontType(0);
      oled.print(Tmax);
  
    if (record!=0) {
      oled.setCursor(0, 30);
      oled.setFontType(0);
      oled.print("n:  ");
      oled.setFontType(0);
      oled.print(EEAdress);
    }
      oled.display();
  }
  else{
      oled.clear(PAGE);            // Clear the display
      oled.setCursor(0, 0);        // Set cursor to top-left
      oled.setFontType(0);         // Smallest font
      oled.print("ERROR:");        
      oled.setCursor(0, 10);        // Set cursor to top-left
      oled.setFontType(0);         // Smallest font
      oled.print("Sensor NOK");        
      oled.display();
  }
  
    data=Serial.readStringUntil('\n');
    if(!data.compareTo("?")) DumpLog();
}


void DumpLog(){
  byte value;
  Serial.println("------Dump Start-------------------------");
  do{
    if (EEAdress<1024) EEAdress++;
    value =  (byte)EEPROM.read(EEAdress);    
    if (value!=0xFF){
      Serial.print(EEAdress);
      Serial.print(";");
      Serial.println(value);
    } 
  }
  while (value!=0xFF);
  EEAdress=0;
  Serial.println("------Dump End-------------------------");
}

void clicked(){
  noInterrupts();
  record=!record;
  digitalWrite(PIN_LED, record); //Control LED
  delay(1500);
  interrupts();
}



